# This lists patterns for camcorder proxy clips. a ';' separated list with
# * relative path from clip to proxy
# * prefix
# * suffix
# * relative path from proxy to clip
# * prefix
# * suffix

# * This sequence of 6 parameters can be repeated to offer several naming schemes

[proxy]
Sony PXW=../Sub;;S03.MP4;../Clip;;.MXF
GoPro LRV=./;GL;.LRV;./;GX;.MP4;./;GP;.LRV;./;GP;.MP4;./;GOPR;.LRV;./;GOPR;.MP4;./;GL;.LRV;./;GH;.MP4
Akaso LRV=./;;.LRV;./;;.MP4
DJI LRF=./;;.LRF;./;;.MP4
Insta360 AcePro LRV=./;LRV;.lrv;./;VID;.mp4;./;PRO_LRV;.lrv;./;PRO_VID;.mp4